"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class AlgorithmBase {
    get best() {
        return this.population.best;
    }
    get progress() {
        return this.termination.progress;
    }
    get hasReached() {
        return this.termination.hasReached(this);
    }
    reset() {
        return __awaiter(this, void 0, void 0, function* () {
            this.population.init();
            this.termination.init();
        });
    }
    step() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.hasReached) {
                return;
            }
            const { parents, population } = yield this.selection.select(this.population.chromosomes);
            const offspring = yield this.crossover.cross(parents, this.crossoverProbability);
            yield Promise.all(offspring.map(chromosome => () => __awaiter(this, void 0, void 0, function* () {
                this.mutation.mutate(chromosome, this.mutationProbability);
                chromosome.fitness = yield this.fitness.evaluate(chromosome);
            })));
            yield Promise.all(parents.map(chromosome => () => __awaiter(this, void 0, void 0, function* () {
                var _a;
                chromosome.fitness = (_a = chromosome.fitness) !== null && _a !== void 0 ? _a : yield this.fitness.evaluate(chromosome);
            })));
            const newGeneration = yield this.reinsertion.select(population, offspring, parents, this.population.size);
            this.population.update(newGeneration);
        });
    }
}
exports.AlgorithmBase = AlgorithmBase;
//# sourceMappingURL=AlgorithmBase.js.map